@@ -6,7 +6,7 @@ Once you understand this example you'll know the basics of OpenID
 and using the Python OpenID library. You can then move on to more
 robust examples, and integrating OpenID into your application.
 """
-__copyright__ = 'Copyright 2005, Janrain, Inc.'
+__copyright__ = 'Copyright 2005-2007, Janrain, Inc.'
 
 from Cookie import SimpleCookie
 import cgi
@@ -23,7 +23,7 @@ from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
 try:
     import openid
 except ImportError:
-    print >>sys.stderr, """
+    sys.stderr.write("""
 Failed to import the OpenID library. In order to use this example, you
 must either install the library (see INSTALL in the root of the
 distribution) or else add the library to python's import path (the
@@ -31,15 +31,21 @@ PYTHONPATH environment variable).
 
 For more information, see the README in the root of the library
 distribution or http://www.openidenabled.com/
-"""
+""")
     sys.exit(1)
 
+from openid.store import memstore
 from openid.store import filestore
 from openid.consumer import consumer
 from openid.oidutil import appendArgs
 from openid.cryptutil import randomString
-from yadis.discover import DiscoveryFailure
-from urljr.fetchers import HTTPFetchingError
+from openid.fetchers import setDefaultFetcher, Urllib2Fetcher
+from openid import sreg
+
+# Used with an OpenID provider affiliate program.
+OPENID_PROVIDER_NAME = 'MyOpenID'
+OPENID_PROVIDER_URL ='https://www.myopenid.com/affiliate_signup?affiliate_id=39'
+
 
 class OpenIDHTTPServer(HTTPServer):
     """http server that contains a reference to an OpenID consumer and
@@ -120,7 +126,7 @@ class OpenIDRequestHandler(BaseHTTPRequestHandler):
             self.parsed_uri = urlparse.urlparse(self.path)
             self.query = {}
             for k, v in cgi.parse_qsl(self.parsed_uri[4]):
-                self.query[k] = v
+                self.query[k] = v.decode('utf-8')
 
             path = self.parsed_uri[2]
             if path == '/':
@@ -129,6 +135,8 @@ class OpenIDRequestHandler(BaseHTTPRequestHandler):
                 self.doVerify()
             elif path == '/process':
                 self.doProcess()
+            elif path == '/affiliate':
+                self.doAffiliate()
             else:
                 self.notFound()
 
@@ -152,16 +160,13 @@ class OpenIDRequestHandler(BaseHTTPRequestHandler):
                         css_class='error', form_contents=openid_url)
             return
 
+        immediate = 'immediate' in self.query
+        use_sreg = 'use_sreg' in self.query
+
         oidconsumer = self.getConsumer()
         try:
             request = oidconsumer.begin(openid_url)
-        except HTTPFetchingError, exc:
-            fetch_error_string = 'Error in discovery: %s' % (
-                cgi.escape(str(exc.why)))
-            self.render(fetch_error_string,
-                        css_class='error',
-                        form_contents=openid_url)
-        except DiscoveryFailure, exc:
+        except consumer.DiscoveryFailure, exc:
             fetch_error_string = 'Error in discovery: %s' % (
                 cgi.escape(str(exc[0])))
             self.render(fetch_error_string,
@@ -177,12 +182,30 @@ class OpenIDRequestHandler(BaseHTTPRequestHandler):
                 # Here we find out the identity server that will verify the
                 # user's identity, and get a token that allows us to
                 # communicate securely with the identity server.
+                if use_sreg:
+                    self.requestRegistrationData(request)
 
                 trust_root = self.server.base_url
                 return_to = self.buildURL('process')
-                redirect_url = request.redirectURL(trust_root, return_to)
+                if request.shouldSendRedirect():
+                    redirect_url = request.redirectURL(
+                        trust_root, return_to, immediate=immediate)
+                    self.send_response(302)
+                    self.send_header('Location', redirect_url)
+                    self.writeUserHeader()
+                    self.end_headers()
+                else:
+                    form_html = request.formMarkup(
+                        trust_root, return_to,
+                        form_tag_attrs={'id':'openid_message'},
+                        immediate=immediate)
 
-                self.redirect(redirect_url)
+                    self.autoSubmit(form_html, 'openid_message')
+
+    def requestRegistrationData(self, request):
+        sreg_request = sreg.SRegRequest(
+            required=['nickname'], optional=['fullname', 'email'])
+        request.addExtension(sreg_request)
 
     def doProcess(self):
         """Handle the redirect from the OpenID server.
@@ -195,13 +218,15 @@ class OpenIDRequestHandler(BaseHTTPRequestHandler):
         # the return type.
         info = oidconsumer.complete(self.query)
 
+        sreg_resp = None
         css_class = 'error'
         if info.status == consumer.FAILURE and info.identity_url:
             # In the case of failure, if info is non-None, it is the
             # URL that we were verifying. We include it in the error
             # message to help the user figure out what happened.
-            fmt = "Verification of %s failed."
-            message = fmt % (cgi.escape(info.identity_url),)
+            fmt = "Verification of %s failed: %s"
+            message = fmt % (cgi.escape(info.identity_url),
+                             info.message)
         elif info.status == consumer.SUCCESS:
             # Success means that the transaction completed without
             # error. If info is None, it means that the user cancelled
@@ -213,6 +238,7 @@ class OpenIDRequestHandler(BaseHTTPRequestHandler):
             # comment posting, etc. here.
             fmt = "You have successfully verified %s as your identity."
             message = fmt % (cgi.escape(info.identity_url),)
+            sreg_resp = sreg.SRegResponse.fromSuccessResponse(info)
             if info.endpoint.canonicalID:
                 # You should authorize i-name users by their canonicalID,
                 # rather than their more human-friendly identifiers.  That
@@ -223,6 +249,14 @@ class OpenIDRequestHandler(BaseHTTPRequestHandler):
         elif info.status == consumer.CANCEL:
             # cancelled
             message = 'Verification cancelled'
+        elif info.status == consumer.SETUP_NEEDED:
+            if info.setup_url:
+                message = '<a href=%s>Setup needed</a>' % (
+                    quoteattr(info.setup_url),)
+            else:
+                # This means auth didn't succeed, but you're welcome to try
+                # non-immediate mode.
+                message = 'Setup needed'
         else:
             # Either we don't understand the code or there is no
             # openid_url included with the error. Give a generic
@@ -230,7 +264,42 @@ class OpenIDRequestHandler(BaseHTTPRequestHandler):
             # information in a log.
             message = 'Verification failed.'
 
-        self.render(message, css_class, info.identity_url)
+        self.render(message, css_class, info.identity_url, sreg_data=sreg_resp)
+
+    def doAffiliate(self):
+        """Direct the user sign up with an affiliate OpenID provider."""
+        sreg_req = sreg.SRegRequest(['nickname'], ['fullname', 'email'])
+        href = sreg_req.toMessage().toURL(OPENID_PROVIDER_URL)
+
+        message = """Get an OpenID at <a href=%s>%s</a>""" % (
+            quoteattr(href), OPENID_PROVIDER_NAME)
+        self.render(message)
+
+    def renderSREG(self, sreg_data):
+        if not sreg_data:
+            self.wfile.write(
+                '<div class="alert">No registration data was returned</div>')
+        else:
+            sreg_list = sreg_data.items()
+            sreg_list.sort()
+            self.wfile.write(
+                '<h2>Registration Data</h2>'
+                '<table class="sreg">'
+                '<thead><tr><th>Field</th><th>Value</th></tr></thead>'
+                '<tbody>')
+
+            odd = ' class="odd"'
+            for k, v in sreg_list:
+                field_name = sreg.data_fields.get(k, k)
+                value = cgi.escape(v)
+                self.wfile.write(
+                    '<tr%s><td>%s</td><td>%s</td></tr>' % (odd, field_name, value))
+                if odd:
+                    odd = ''
+                else:
+                    odd = ' class="odd"'
+
+            self.wfile.write('</tbody></table>')
 
     def buildURL(self, action, **query):
         """Build a URL relative to the server base_url, with the given
@@ -238,15 +307,14 @@ class OpenIDRequestHandler(BaseHTTPRequestHandler):
         base = urlparse.urljoin(self.server.base_url, action)
         return appendArgs(base, query)
 
-    def redirect(self, redirect_url):
-        """Send a redirect response to the given URL to the browser."""
+    def autoSubmit(self, form, id):
+        """Send a page containing an auto-submitting form."""
         response = """\
-Location: %s
-Content-type: text/plain
-
-Redirecting to %s""" % (redirect_url, redirect_url)
-        self.send_response(302)
-        self.setSessionCookie()
+<html><head><title>OpenID transaction in progress</title></head>
+<body onload='document.getElementById("%s").submit()'>
+%s
+</body></html>
+"""%(id, form)
         self.wfile.write(response)
 
     def notFound(self):
@@ -257,7 +325,8 @@ Redirecting to %s""" % (redirect_url, redirect_url)
         self.render(msg, 'error', openid_url, status=404)
 
     def render(self, message=None, css_class='alert', form_contents=None,
-               status=200, title="Python OpenID Consumer Example"):
+               status=200, title="Python OpenID Consumer Example",
+               sreg_data=None):
         """Render a page."""
         self.send_response(status)
         self.pageHeader(title)
@@ -265,6 +334,10 @@ Redirecting to %s""" % (redirect_url, redirect_url)
             self.wfile.write("<div class='%s'>" % (css_class,))
             self.wfile.write(message)
             self.wfile.write("</div>")
+
+        if sreg_data is not None:
+            self.renderSREG(sreg_data)
+
         self.pageFooter(form_contents)
 
     def pageHeader(self, title):
@@ -286,6 +359,20 @@ Content-type: text/html
       div {
         padding: .5em;
       }
+      tr.odd td {
+        background-color: #dddddd;
+      }
+      table.sreg {
+        border: 1px solid black;
+        border-collapse: collapse;
+      }
+      table.sreg th {
+        border-bottom: 1px solid black;
+      }
+      table.sreg td, table.sreg th {
+        padding: 0.5em;
+        text-align: left;
+      }
       table {
         margin: none;
         padding: none;
@@ -322,21 +409,29 @@ Content-type: text/html
 
         self.wfile.write('''\
     <div id="verify-form">
-      <form method="get" action=%s>
+      <form method="get" accept-charset="UTF-8" action=%s>
         Identifier:
         <input type="text" name="openid_identifier" value=%s />
-        <input type="submit" value="Verify" />
+        <input type="submit" value="Verify" /><br />
+        <input type="checkbox" name="immediate" id="immediate" /><label for="immediate">Use immediate mode</label>
+        <input type="checkbox" name="use_sreg" id="use_sreg" /><label for="use_sreg">Request registration data</label>
       </form>
     </div>
   </body>
 </html>
 ''' % (quoteattr(self.buildURL('verify')), quoteattr(form_contents)))
 
-def main(host, port, data_path):
+def main(host, port, data_path, weak_ssl=False):
     # Instantiate OpenID consumer store and OpenID consumer.  If you
     # were connecting to a database, you would create the database
     # connection and instantiate an appropriate store here.
-    store = filestore.FileOpenIDStore(data_path)
+    if data_path:
+        store = filestore.FileOpenIDStore(data_path)
+    else:
+        store = memstore.MemoryStore()
+
+    if weak_ssl:
+        setDefaultFetcher(Urllib2Fetcher())
 
     addr = (host, port)
     server = OpenIDHTTPServer(store, addr, OpenIDRequestHandler)
@@ -347,8 +442,8 @@ def main(host, port, data_path):
 
 if __name__ == '__main__':
     host = 'localhost'
-    data_path = 'cstore'
     port = 8001
+    weak_ssl = False
 
     try:
         import optparse
@@ -357,9 +452,9 @@ if __name__ == '__main__':
     else:
         parser = optparse.OptionParser('Usage:\n %prog [options]')
         parser.add_option(
-            '-d', '--data-path', dest='data_path', default=data_path,
+            '-d', '--data-path', dest='data_path',
             help='Data directory for storing OpenID consumer state. '
-            'Defaults to "%default" in the current directory.')
+            'Setting this option implies using a "FileStore."')
         parser.add_option(
             '-p', '--port', dest='port', type='int', default=port,
             help='Port on which to listen for HTTP requests. '
@@ -368,6 +463,9 @@ if __name__ == '__main__':
             '-s', '--host', dest='host', default=host,
             help='Host on which to listen for HTTP requests. '
             'Also used for generating URLs. Defaults to %default.')
+        parser.add_option(
+            '-w', '--weakssl', dest='weakssl', default=False,
+            action='store_true', help='Skip ssl cert verification')
 
         options, args = parser.parse_args()
         if args:
@@ -376,5 +474,6 @@ if __name__ == '__main__':
         host = options.host
         port = options.port
         data_path = options.data_path
+        weak_ssl = options.weakssl
 
-    main(host, port, data_path)
+    main(host, port, data_path, weak_ssl)
