@@ -18,7 +18,7 @@ def quoteattr(s):
 try:
     import openid
 except ImportError:
-    print >>sys.stderr, """
+    sys.stderr.write("""
 Failed to import the OpenID library. In order to use this example, you
 must either install the library (see INSTALL in the root of the
 distribution) or else add the library to python's import path (the
@@ -26,19 +26,20 @@ PYTHONPATH environment variable).
 
 For more information, see the README in the root of the library
 distribution or http://www.openidenabled.com/
-"""
+""")
     sys.exit(1)
 
-from openid import oidutil
+from openid import sreg
 from openid.server import server
 from openid.store.filestore import FileOpenIDStore
+from openid.consumer import discover
 
 class OpenIDHTTPServer(HTTPServer):
     """
     http server that contains a reference to an OpenID Server and
     knows its base URL.
     """
-    def __init__(self, oidserver, *args, **kwargs):
+    def __init__(self, *args, **kwargs):
         HTTPServer.__init__(self, *args, **kwargs)
 
         if self.server_port != 80:
@@ -47,10 +48,14 @@ class OpenIDHTTPServer(HTTPServer):
         else:
             self.base_url = 'http://%s/' % (self.server_name,)
 
-        self.openid = oidserver
+        self.openid = None
         self.approved = {}
         self.lastCheckIDRequest = {}
 
+    def setOpenIDServer(self, oidserver):
+        self.openid = oidserver
+
+
 class ServerHandler(BaseHTTPRequestHandler):
     def __init__(self, *args, **kwargs):
         self.user = None
@@ -77,8 +82,15 @@ class ServerHandler(BaseHTTPRequestHandler):
                 self.showLoginPage('/', '/')
             elif path == '/loginsubmit':
                 self.doLogin()
-            else:
+            elif path.startswith('/id/'):
                 self.showIdPage(path)
+            elif path.startswith('/yadis/'):
+                self.showYadis(path[7:])
+            elif path == '/serveryadis':
+                self.showServerYadis()
+            else:
+                self.send_response(404)
+                self.end_headers()
 
         except (KeyboardInterrupt, SystemExit):
             raise
@@ -124,19 +136,19 @@ class ServerHandler(BaseHTTPRequestHandler):
         request = self.server.lastCheckIDRequest.get(self.user)
 
         if 'yes' in query:
-            identity = request.identity
-            trust_root = request.trust_root
-            if self.query.get('remember', 'no') == 'yes':
-                duration = 'always'
-            else:
-                duration = 'once'
-
-            self.server.approved[(identity, trust_root)] = duration
-
             if 'login_as' in query:
                 self.user = self.query['login_as']
 
-            response = request.answer(True)
+            if request.idSelect():
+                identity = self.server.base_url + 'id/' + query['identifier']
+            else:
+                identity = request.identity
+
+            trust_root = request.trust_root
+            if self.query.get('remember', 'no') == 'yes':
+                self.server.approved[(identity, trust_root)] = 'always'
+
+            response = self.approved(request, identity)
 
         elif 'no' in query:
             response = request.answer(False)
@@ -162,11 +174,7 @@ class ServerHandler(BaseHTTPRequestHandler):
             return False
 
         key = (identity_url, trust_root)
-        approval = self.server.approved.get(key)
-        if approval == 'once':
-            del self.server.approved[key]
-
-        return approval is not None
+        return self.server.approved.get(key) is not None
 
     def serverEndPoint(self, query):
         try:
@@ -181,19 +189,40 @@ class ServerHandler(BaseHTTPRequestHandler):
             return
 
         if request.mode in ["checkid_immediate", "checkid_setup"]:
-            if (self.user and
-                self.isAuthorized(request.identity, request.trust_root)):
-                response = request.answer(True)
-            elif request.immediate:
-                response = request.answer(
-                    False, server_url=self.server.base_url + 'openidserver')
-            else:
-                self.server.lastCheckIDRequest[self.user] = request
-                self.showDecidePage(request)
-                return
+            self.handleCheckIDRequest(request)
         else:
             response = self.server.openid.handleRequest(request)
-        self.displayResponse(response)
+            self.displayResponse(response)
+
+    def addSRegResponse(self, response):
+        sreg_req = sreg.SRegRequest.fromOpenIDRequest(request.message)
+
+        # In a real application, this data would be user-specific,
+        # and the user should be asked for permission to release
+        # it.
+        sreg_data = {
+            'nickname':self.user
+            }
+
+        sreg_resp = sreg.SRegResponse.extractResponse(sreg_req, sreg_data)
+        response.addExtension(sreg_resp)
+
+    def approved(self, request, identifier=None):
+        response = request.answer(True, identity=identifier)
+        self.addSRegResponse(response)
+        return response
+
+    def handleCheckIDRequest(self, request):
+        is_authorized = self.isAuthorized(request.identity, request.trust_root)
+        if is_authorized:
+            response = self.approved(request)
+            self.displayResponse(response)
+        elif request.immediate:
+            response = request.answer(False)
+            self.displayResponse(response)
+        else:
+            self.server.lastCheckIDRequest[self.user] = request
+            self.showDecidePage(request)
 
     def displayResponse(self, response):
         try:
@@ -305,11 +334,40 @@ class ServerHandler(BaseHTTPRequestHandler):
         ''' % error_message)
 
     def showDecidePage(self, request):
-        expected_user = request.identity[len(self.server.base_url):]
+        id_url_base = self.server.base_url+'id/'
+        expected_user = request.identity[len(id_url_base):]
 
-        if expected_user == self.user:
+        if request.idSelect(): # We are being asked to select an ID
             msg = '''\
-            <p>A new site has asked for your identity.  If you
+            <p>A site has asked for your identity.  You may select an
+            identifier by which you would like this site to know you.
+            On a production site this would likely be a drop down list
+            of pre-created accounts or have the facility to generate
+            a random anonymous identifier.
+            </p>
+            '''
+            fdata = {
+                'id_url_base': id_url_base,
+                'trust_root': request.trust_root,
+                }
+            form = '''\
+            <form method="POST" action="/allow">
+            <table>
+              <tr><td>Identity:</td>
+                 <td>%(id_url_base)s<input type='text' name='identifier'></td></tr>
+              <tr><td>Trust Root:</td><td>%(trust_root)s</td></tr>
+            </table>
+            <p>Allow this authentication to proceed?</p>
+            <input type="checkbox" id="remember" name="remember" value="yes"
+                /><label for="remember">Remember this
+                decision</label><br />
+            <input type="submit" name="yes" value="yes" />
+            <input type="submit" name="no" value="no" />
+            </form>
+            '''%fdata
+        elif expected_user == self.user:
+            msg = '''\
+            <p>A new site has asked to confirm your identity.  If you
             approve, the site represented by the trust root below will
             be told that you control identity URL listed below. (If
             you are using a delegated identity, the site will take
@@ -327,7 +385,7 @@ class ServerHandler(BaseHTTPRequestHandler):
             <p>Allow this authentication to proceed?</p>
             <form method="POST" action="/allow">
               <input type="checkbox" id="remember" name="remember" value="yes"
-                  checked="checked" /><label for="remember">Remember this
+                  /><label for="remember">Remember this
                   decision</label><br />
               <input type="submit" name="yes" value="yes" />
               <input type="submit" name="no" value="no" />
@@ -357,7 +415,7 @@ class ServerHandler(BaseHTTPRequestHandler):
             <p>Allow this authentication to proceed?</p>
             <form method="POST" action="/allow">
               <input type="checkbox" id="remember" name="remember" value="yes"
-                  checked="checked" /><label for="remember">Remember this
+                  /><label for="remember">Remember this
                   decision</label><br />
               <input type="hidden" name="login_as" value="%(expected_user)s"/>
               <input type="submit" name="yes" value="yes" />
@@ -367,8 +425,11 @@ class ServerHandler(BaseHTTPRequestHandler):
         self.showPage(200, 'Approve OpenID request?', msg=msg, form=form)
 
     def showIdPage(self, path):
-        tag = '<link rel="openid.server" href="%sopenidserver">' %\
+        link_tag = '<link rel="openid.server" href="%sopenidserver">' %\
               self.server.base_url
+        yadis_loc_tag = '<meta http-equiv="x-xrds-location" content="%s">'%\
+            (self.server.base_url+'yadis/'+path[4:])
+        disco_tags = link_tag + yadis_loc_tag
         ident = self.server.base_url + path[1:]
 
         approved_trust_roots = []
@@ -385,14 +446,64 @@ class ServerHandler(BaseHTTPRequestHandler):
         else:
             msg = ''
 
-        self.showPage(200, 'An Identity Page', link_tag=tag, msg='''\
+        self.showPage(200, 'An Identity Page', head_extras=disco_tags, msg='''\
         <p>This is an identity page for %s.</p>
         %s
         ''' % (ident, msg))
 
+    def showYadis(self, user):
+        self.send_response(200)
+        self.send_header('Content-type', 'application/xrds+xml')
+        self.end_headers()
+
+        endpoint_url = self.server.base_url + 'openidserver'
+        user_url = self.server.base_url + 'id/' + user
+        self.wfile.write("""\
+<?xml version="1.0" encoding="UTF-8"?>
+<xrds:XRDS
+    xmlns:xrds="xri://$xrds"
+    xmlns="xri://$xrd*($v*2.0)">
+  <XRD>
+
+    <Service priority="0">
+      <Type>%s</Type>
+      <Type>%s</Type>
+      <URI>%s</URI>
+      <LocalID>%s</LocalID>
+    </Service>
+
+  </XRD>
+</xrds:XRDS>
+"""%(discover.OPENID_2_0_TYPE, discover.OPENID_1_0_TYPE,
+     endpoint_url, user_url))
+
+    def showServerYadis(self):
+        self.send_response(200)
+        self.send_header('Content-type', 'application/xrds+xml')
+        self.end_headers()
+
+        endpoint_url = self.server.base_url + 'openidserver'
+        self.wfile.write("""\
+<?xml version="1.0" encoding="UTF-8"?>
+<xrds:XRDS
+    xmlns:xrds="xri://$xrds"
+    xmlns="xri://$xrd*($v*2.0)">
+  <XRD>
+
+    <Service priority="0">
+      <Type>%s</Type>
+      <URI>%s</URI>
+    </Service>
+
+  </XRD>
+</xrds:XRDS>
+"""%(discover.OPENID_IDP_2_0_TYPE, endpoint_url,))
+
     def showMainPage(self):
+        yadis_tag = '<meta http-equiv="x-xrds-location" content="%s">'%\
+            (self.server.base_url + 'serveryadis')
         if self.user:
-            openid_url = self.server.base_url + self.user
+            openid_url = self.server.base_url + 'id/' + self.user
             user_message = """\
             <p>You are logged in as %s. Your OpenID identity URL is
             <tt><a href=%s>%s</a></tt>. Enter that URL at an OpenID
@@ -404,7 +515,7 @@ class ServerHandler(BaseHTTPRequestHandler):
             order to simulate a standard Web user experience. You are
             not <a href='/login'>logged in</a>.</p>"""
 
-        self.showPage(200, 'Main Page', msg='''\
+        self.showPage(200, 'Main Page', head_extras = yadis_tag, msg='''\
         <p>This is a simple OpenID server implemented using the <a
         href="http://openid.schtuff.com/">Python OpenID
         library</a>.</p>
@@ -435,7 +546,7 @@ class ServerHandler(BaseHTTPRequestHandler):
         ''' % (success_to, fail_to))
 
     def showPage(self, response_code, title,
-                 link_tag='', msg=None, err=None, form=None):
+                 head_extras='', msg=None, err=None, form=None):
 
         if self.user is None:
             user_link = '<a href="/login">not logged in</a>.'
@@ -468,21 +579,20 @@ class ServerHandler(BaseHTTPRequestHandler):
 
         contents = {
             'title': 'Python OpenID Server Example - ' + title,
-            'link_tag': link_tag,
+            'head_extras': head_extras,
             'body': body,
             'user_link': user_link,
             }
 
         self.send_response(response_code)
         self.writeUserHeader()
+        self.send_header('Content-type', 'text/html')
+        self.end_headers()
 
-        self.wfile.write('''\
-Content-type: text/html
-
-<html>
+        self.wfile.write('''<html>
   <head>
     <title>%(title)s</title>
-    %(link_tag)s
+    %(head_extras)s
   </head>
   <style type="text/css">
       h1 a:link {
@@ -558,18 +668,16 @@ Content-type: text/html
 
 
 def main(host, port, data_path):
+    addr = (host, port)
+    httpserver = OpenIDHTTPServer(addr, ServerHandler)
+
     # Instantiate OpenID consumer store and OpenID consumer.  If you
     # were connecting to a database, you would create the database
     # connection and instantiate an appropriate store here.
-    if port == 80:
-        server_url = 'http://%s/openidserver' % host
-    else:
-        server_url = 'http://%s:%s/openidserver' % (host, port)
     store = FileOpenIDStore(data_path)
-    oidserver = server.Server(store)
+    oidserver = server.Server(store, httpserver.base_url + 'openidserver')
 
-    addr = (host, port)
-    httpserver = OpenIDHTTPServer(oidserver, addr, ServerHandler)
+    httpserver.setOpenIDServer(oidserver)
 
     print 'Server running at:'
     print httpserver.base_url
