@@ -14,15 +14,60 @@ http://www.amk.ca/python/code/crypto
 """
 
 __all__ = ['randrange', 'hmacSha1', 'sha1', 'randomString',
-           'binaryToLong', 'longToBinary', 'longToBase64', 'base64ToLong']
+           'binaryToLong', 'longToBinary', 'longToBase64', 'base64ToLong',
+           'hmacSha256', 'sha256']
 
 import hmac
 import os
 import random
-import sha
 
 from openid.oidutil import toBase64, fromBase64
 
+try:
+    import hashlib
+except ImportError:
+    import sha as sha1_module
+
+    try:
+        from Crypto.Hash import SHA256 as sha256_module
+    except ImportError:
+        sha256_module = None
+
+else:
+    class HashContainer(object):
+        def __init__(self, hash_constructor):
+            self.new = hash_constructor
+
+    sha1_module = HashContainer(hashlib.sha1)
+    sha256_module = HashContainer(hashlib.sha256)
+
+def hmacSha1(key, text):
+    return hmac.new(key, text, sha1_module).digest()
+
+def sha1(s):
+    return sha1_module.new(s).digest()
+
+if sha256_module is not None:
+    def hmacSha256(key, text):
+        return hmac.new(key, text, sha256_module).digest()
+
+    def sha256(s):
+        return sha256_module.new(s).digest()
+
+    SHA256_AVAILABLE = True
+
+else:
+    _no_sha256 = NotImplementedError(
+        'Use Python 2.5, install pycrypto or install hashlib to use SHA256')
+
+    def hmacSha256(unused_key, unused_text):
+        raise _no_sha256
+
+    def sha256(s):
+        raise _no_sha256
+
+    SHA256_AVAILABLE = False
+
 try:
     from Crypto.Util.number import long_to_bytes, bytes_to_long
 except ImportError:
@@ -150,12 +195,6 @@ except AttributeError:
 
         return start + (n % r) * step
 
-def hmacSha1(key, text):
-    return hmac.new(key, text, sha).digest()
-
-def sha1(s):
-    return sha.new(s).digest()
-
 def longToBase64(l):
     return toBase64(longToBinary(l))
 
