@@ -13,7 +13,7 @@ _top_level_domains = (
     'com|edu|gov|int|mil|net|org|biz|info|name|museum|coop|aero|ac|ad|ae|'
     'af|ag|ai|al|am|an|ao|aq|ar|as|at|au|aw|az|ba|bb|bd|be|bf|bg|bh|bi|bj|'
     'bm|bn|bo|br|bs|bt|bv|bw|by|bz|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|co|cr|'
-    'cu|cv|cx|cy|cz|de|dj|dk|dm|do|dz|ec|ee|eg|eh|er|es|et|fi|fj|fk|fm|fo|'
+    'cu|cv|cx|cy|cz|de|dj|dk|dm|do|dz|ec|ee|eg|eh|er|es|et|eu|fi|fj|fk|fm|fo|'
     'fr|ga|gd|ge|gf|gg|gh|gi|gl|gm|gn|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|'
     'ht|hu|id|ie|il|im|in|io|iq|ir|is|it|je|jm|jo|jp|ke|kg|kh|ki|km|kn|kp|'
     'kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|mg|mh|mk|ml|mm|'
@@ -27,6 +27,13 @@ _top_level_domains = (
 
 def _parseURL(url):
     proto, netloc, path, params, query, frag = urlparse(url)
+    if not path:
+        # Python <2.4 does not parse URLs with no path properly
+        if not query and '?' in netloc:
+            netloc, query = netloc.split('?', 1)
+
+        path = '/'
+
     path = urlunparse(('', '', path, params, query, frag))
 
     if ':' in netloc:
@@ -39,9 +46,6 @@ def _parseURL(url):
         port = ''
 
     host = host.lower()
-    if not path:
-        path = '/'
-
     return proto, host, port, path
 
 class TrustRoot(object):
@@ -105,25 +109,18 @@ class TrustRoot(object):
         if tld not in _top_level_domains:
             return False
 
-        if len(tld) == 2:
-            if len(host_parts) == 1:
-                # entire host part is 2-letter tld
-                return False
+        if len(host_parts) == 1:
+            return False
 
-            if len(host_parts[-2]) <= 3:
+        if self.wildcard:
+            if len(tld) == 2 and len(host_parts[-2]) <= 3:
                 # It's a 2-letter tld with a short second to last segment
                 # so there needs to be more than two segments specified 
                 # (e.g. *.co.uk is insane)
                 return len(host_parts) > 2
-            else:
-                # A long second to last segment is specified.
-                return len(host_parts) > 1
-        else:
-            # It's a regular tld, so it needs at least one more segment
-            return len(host_parts) > 1
 
-        # Fell through, so not sane
-        return False
+        # Passed all tests for insanity.
+        return True
 
     def validateURL(self, url):
         """
@@ -214,6 +211,10 @@ class TrustRoot(object):
         if proto not in _protocols:
             return None
 
+        # check for URI fragment
+        if path.find('#') != -1:
+            return None
+
         # extract wildcard if it is there
         if host.find('*', 1) != -1:
             # wildcard must be at start of domain:  *.foo.com, not foo.*.com
